#!/bin/bash

###BEGIN variable define###
uos=0
kylin=0
nfs=0
error=0
result1_num=0
line_start=0
line_end=0
result2_not=0
backup_not=0
restore_not=0
os_find=0


test_path=/usr/sugon-recovery
boot_path=/boot/efi/boot
output_path=/boot/efi/boot/recovery

grub_path=$test_path/os.txt
result=$test_path/test.cfg
result1=$test_path/test1.txt
result2=$test_path/test2.txt
result3=$test_path/test3.txt

backup=$test_path/backup.txt
restore=$test_path/restore.txt
tmp=$test_path/tmp.txt

recovery=$test_path/recovery.cfg

boot_grub=/boot/grub/grub.cfg

output_file=/boot/efi/boot/recovery/recovery.cfg
efi_file=/boot/efi/boot/recovery/recovery.efi
source_directory=$(dirname $0)
###END variable define###

###BEGIN Function define###
getostype_fun() {
find /boot/efi/efi/ -name grubx64.efi > ${grub_path}
cat ${grub_path} | grep "kylin"
if [ $? -eq 0 ];then
  kylin=1
  os_grub=/boot/efi/efi/kylin/grub.cfg
fi

cat ${grub_path} | grep "uos"
if [ $? -eq 0 ];then
  uos=1
  os_grub=/boot/efi/efi/uos/grub.cfg
else
  cat ${grub_path} | grep "UOS"
  if [ $? -eq 0 ];then
    uos=1
    os_grub=/boot/efi/efi/uos/grub.cfg 
  fi
fi

cat ${grub_path} | grep "nfs"
if [ $? -eq 0 ];then
  nfs=1
  os_grub=/boot/efi/efi/nfs/grub.cfg
else
  cat ${grub_path} | grep "NFS"
  if [ $? -eq 0 ];then
    nfs=1
    os_grub=/boot/efi/efi/nfs/grub.cfg 
  fi
fi
}

getbackup() {
cat ${result} | grep " backup" > ${backup}
if [ -s ${backup} ];then
  echo "backup is found"
else
  if [ -s ${boot_grub} ];then
    cat ${boot_grub} | grep " backup" > ${backup}
    if [ -s ${backup} ];then
      echo "backup is found"
    else
      if [ -s ${os_grub} ];then
        cat ${os_grub} | grep " backup" > ${backup}
        if [ -s ${backup} ];then
          echo "backup is found"
        else
          backup_not=1
        fi
      fi
    fi
  fi
fi
}

getrestore() {
cat ${result} | grep " restore" > ${restore}
if [ -s ${restore} ];then
  echo "restore is found"
else
  if [ -s ${boot_grub} ];then
    cat ${boot_grub} | grep " restore" > ${restore}
    if [ -s ${restore} ];then
      echo "restore is found"
    else
      if [ -s ${os_grub} ];then
        cat ${os_grub} | grep " restore" > ${restore}
        if [ -s ${restore} ];then
          echo "restore is found"
        else
          restore_not=1
        fi
      fi
    fi
  fi
fi
}

getfilesystem() {
cat ${result} | grep filesystem > ${result2}
if [ -s ${result2} ];then
  echo "linux boot variable is found"
else
  if [ -s ${boot_grub} ];then
    cat ${boot_grub} | grep filesystem > ${result2}
    if [ -s ${result2} ];then
      echo "linux boot variable is found" 
    else
      if [ -s ${os_grub} ];then
        cat ${os_grub} | grep filesystem > ${result2}
        if [ -s ${result2} ];then
          echo "linux boot variable is found" 
        else
          result2_not=1
        fi
      fi
    fi
  fi
fi
}

get_os_prober() {
echo "get 30-os-prober"

line_start="`cat ${result} | grep -n "BEGIN /etc/grub.d/30_os-prober" | awk -F ":" '{print $1}'`"
line_end="`cat ${result} | grep -n "END /etc/grub.d/30_os-prober" | awk -F ":" '{print $1}'`"
echo $line_start
echo $line_end

sed -n "${line_start},${line_end}p" ${result} > ${tmp}

if [ -s ${tmp} ]; then
cat ${tmp} | grep "search " > ${result1}
if [ -s ${result1} ];then
  result1_num=`cat ${result1} |wc -l`
else
  echo "error： uuid is not found"
  error=1
fi

cat ${tmp} | grep "root=UUID" > ${result2}
if [ -s ${result2} ];then
  echo "linux image is found"
else
  echo "error： linux image is not found"
  error=1
fi


cat ${tmp} | grep .img > ${result3}
if [ -s ${result3} ];then
  echo "initrd image is found"
else
  echo "error： initrd image is not found"
  error=1
fi
else
error=1

fi

}

justrecovery_header_info() {
cat >>${recovery} <<EOF
###BEGIN /etc/grub.d/01_header###
terminal_output console
if [ x\$feature_timeout_style = xy ]; then
  set timeout_style=hidden
  set timeout=0
else
  set timeout=1
fi
###END /etc/grub.d/01_header###

###BEGIN /etc/grub.d/15_linux_bar###
EOF
}

backup_restore_info() {
cat >>$recovery <<EOF
###BEGIN /etc/grub.d/01_header###
set default="0"

if [ x"\${feature_menuentry_id}" = xy ]; then
  menuentry_id_option="--id"
else
  menuentry_id_option=""
fi
export menuentry_id_option

if [ x\$feature_timeout_style = xy ]; then
  set timeout_style=menu
  set timeout=5
else
  set timeout=5
fi
### END /etc/grub.d/00_header ###

### BEGIN /etc/grub.d/05_debian_theme ###
set linux_gfx_mode=text
export linux_gfx_mode

set menu_color_normal=white/black
set menu_color_highlight=black/white
set color_normal=white/black
set color_highlight=black/white
### END /etc/grub.d/05_debian_theme ###
EOF
}

prepare_menu_info() {
cat >>$recovery <<EOF
  insmod gzio
  insmod lzopio
  insmod part_gpt
  insmod ext2
  set root='hd0,gpt1'
  if [ x\$feature_platform_search_hint = xy ] ; then
EOF
}

backup_info() {
cat >>$recovery <<EOF
###BEGIN /etc/grub.d/15_linux_bar###
menuentry 'Kylin V10 (备份模式)' --class kylin --class gnu-linux --class gnu --class os \$menuentry_id_option 'gnulinux-backup' {
EOF
}

restore_info() {
cat >>$recovery <<EOF
menuentry 'Kylin V10 (还原模式)' --class kylin --class gnu-linux --class gnu --class os \$menuentry_id_option 'gnulinux-restore' {
EOF
}

set_search() {
if [ $result1_num -ge 4 ];then
awk 'NR==3 {print}' ${result1} >>${recovery}
cat >>${recovery} <<EOF
else
EOF
awk 'NR==4 {print}' ${result1} >>${recovery}
cat >>${recovery} <<EOF
fi
EOF
fi
if [ $result1_num -le 3 ];then
awk 'NR==1 {print}' ${result1} >>${recovery}
cat >>${recovery} <<EOF
else
EOF
awk 'NR==2 {print}' ${result1} >>${recovery}
cat >>${recovery} <<EOF
fi
EOF
fi
}

justrecovery_linux() {
if [ -s ${result2} ];then
  awk 'NR==1 {print}' ${result2} >>${recovery}
fi            
if [ -s ${result3} ];then
 awk 'NR==1 {print}' ${result3} >>${recovery} 
fi
cat >>${recovery} <<EOF
boot
###END /etc/grub.d/15_linux_bar###
EOF
}

backup_linux() {
awk 'NR==1 {print}' ${backup} >>${recovery}
          
if [ -s ${result3} ];then
 awk 'NR==1 {print}' ${result3} >>${recovery} 
fi
}

restore_linux() {
awk 'NR==1 {print}' ${restore} >>${recovery}
          
if [ -s ${result3} ];then
awk 'NR==1 {print}' ${result3} >>${recovery} 
fi
}

###END Function define###

#### generate the recovery.cfg######
##rm -f ${test_path}/*
##rmdir ${test_path}
if [ ! -d "${test_path}" ];then
mkdir ${test_path}
fi

getostype_fun

grub-mkconfig -o ${result}
echo "###/boot/recovery###" > ${recovery}

cat ${result} | grep fs-uuid > ${result1}
if [ -s ${result1} ];then
  result1_num=`cat ${result1} |wc -l`
else
  echo "error： uuid is not found"
  error=1
fi

cat ${result} | grep .img > ${result3}
if [ -s ${result3} ];then
  echo "initrd image is found"
else
  echo "error： initrd image is not found"
  error=1
fi

##start uos###
if [ $uos -eq 1 ];then
  echo "system is uos"
  getfilesystem
  if [ $result2_not -eq 0 ];then
    os_find=1
    justrecovery_header_info
    prepare_menu_info
    set_search
    justrecovery_linux
  fi
fi
##end uos###
##start kylin###
if [ $kylin -eq 1 ];then
  echo "system is kylin OS"
  getbackup
  getrestore
  if [ $backup_not -eq 1 -a $restore_not -eq 1 ];then
    os_find=0
  else
    os_find=1
    if [ $backup_not -eq 1 -a $restore_not -eq 0 ];then
      echo "justrestore"
      justrecovery_header_info
      prepare_menu_info
      set_search
      restore_linux
    fi

    if [ $backup_not -eq 0 -a $restore_not -eq 0 ];then
      backup_restore_info
      backup_info
      prepare_menu_info
      set_search
      backup_linux
cat >>${recovery} <<EOF
}
EOF

      restore_info
      prepare_menu_info
      set_search
      restore_linux

cat >>${recovery} <<EOF
}
###END /etc/grub.d/15_linux_bar###
EOF
    fi
  fi  
fi
##end kylin##
##start nfs###
if [ $nfs -eq 1 ];then
  echo "system is nfs"
  get_os_prober
  if [ $error -eq 0 ];then
    justrecovery_header_info
    prepare_menu_info
    set_search
    justrecovery_linux
    os_find=1
  fi
fi
##end nfs##

if [ -d "${output_path}" ];then
  rm -f ${output_path}/*
  rmdir ${output_path}
fi

#### copy the recovery files and print message#####
if [ $backup_not -eq 1 -a $restore_not -eq 1 ];then
  if [ $result2_not -eq 1 ]; then
    error=1
  fi
fi
if [ $error -eq 1 -o $os_find -eq 0 ];then
  echo "error： recovery.cfg is fail"
  exit 1
else
  if [ ! -d "${boot_path}" ];then
    mkdir ${boot_path}
    mkdir ${output_path}
  else
    if [ ! -d "${output_path}" ];then
      mkdir ${output_path}
    fi
  fi
  cp ${recovery} ${output_file}
  cp ${source_directory}/recovery.efi ${efi_file}

  echo "recovery.cfg is ok"
  exit 0
fi

